import random

import pygame

from player import Player
from colpo import Proiettile
from ship import Ship
from mappa import Map
from explode import Boom
from fumo import Fumo
from star import Star

class Main:
	def __init__(self, map, acc, mode, punti):
		# Display
		self.screen = pygame.display.set_mode((800, 640))
		pygame.display.set_caption('SpaceFight')
		
		self.font = pygame.font.Font('fonts/dorado.ttf', 14)
		self.winfont = pygame.font.Font('fonts/dorado.ttf', 36)
		
		# Variabili
		
		# Gioco
		self.mode = mode
		
		#Mappa
		self.map = Map(map)
		
		# Giocatore
		self.acc = acc
		self.player = Player(acc, punti)
		self.punti = punti
		self.playerpos = 340
		# movimento
		self.playermove = 0
		self.shoot = False
		self.shootgap = 0
		
		# Global
		self.proiettili = []
		self.explosions = []
		self.fumi = []
		self.ships = []
		self.stars = []
		self.time = 0
		
		# Gaps
		self.shipgap = 0
		self.fumigap = 0
		
		# Win/Lose
		self.win = False
		self.wintime = 120
		self.active = True
		self.alreadywin = False
		
		# Stars
		for n in range(800):
			if random.randint(0, 2) == 0:
				self.stars.append(Star(random.randint(0, 640), n))
		
		self.run()
	def run(self):
		while True:
			for event in pygame.event.get():
				if event.type == pygame.QUIT:
					exit(0)
				elif event.type == pygame.KEYDOWN:
					if event.key == 273:
						self.playermove = -10
					elif event.key == 274:
						self.playermove = 10
					elif event.key == 32:
						self.shoot = True
				elif event.type == pygame.KEYUP:
					if event.key in (273, 274):
						self.playermove = 0
					elif event.key == 32:
						self.shoot = False
			
			if self.active:
				self.screen.fill((0, 0, 0))
				
				
				# Fumi
				if self.fumigap == 7:
					self.fumigap = 0
				
				for fumo in self.fumi:
					if float(fumo.nsize) == float(fumo.size):
						self.fumi.remove(fumo)
					else:
						if self.fumigap == 0:
							fumo.update()
						
						fumo.draw(self.screen)
				
				
				# Esplosioni
				for boom in self.explosions:
					if boom.nfot == boom.fotogrammi:
						self.explosions.remove(boom)
					else:
						boom.draw(self.screen)
				
				# Stelle
				if random.randint(0, 2) == 0:
					self.stars.append(Star(random.randint(0, 640)))
				
				for star in self.stars:
					star.update()
					
					if star.x < 0:
						self.stars.remove(star)
					else:
						star.draw(self.screen)
				
				# Proiettili
				if self.shoot:
					if self.shootgap == 0:
						self.proiettili.append(Proiettile(1, self.player.weapon1, (90, self.playerpos), 'player'))
				
					self.shootgap += 1
					if self.shootgap == self.player.firegap:
						self.shootgap = 0
			
				for pro in self.proiettili:
					if pro.pos[0] < 900:
						pro.update()
						pro.draw(self.screen)
					
						for ship in self.ships:
							if pro.parent == 'player' and pro.collide(ship.square()):
								ship.danneggia(pro.danno)
								self.explosions.append(Boom(pro.pos, pro.type))
								
								try:
									self.proiettili.remove(pro)
								except:
									pass
								
								if ship.armor <= 0:
									self.player.punti += ship.punti
									self.explosions.append(Boom((ship.x, ship.y), ship.type))
									self.ships.remove(ship)
						
						if pro.parent == 'enemy' and pro.collide((40, self.playerpos-18, 50, 35)):
							self.player.danneggia(pro.danno)
							self.explosions.append(Boom(pro.pos, pro.type))
							self.proiettili.remove(pro)
							
							if self.player.defense <= 0:
								self.active = False
								self.screen.blit(self.winfont.render('You Lose', True, (200, 200, 200)), (530, 530))
					else:
						self.proiettili.remove(pro)
			
			
				# Nemici
				if self.shipgap == 4:
					self.shipgap = 0
				
				for item in self.map.list:
					if int(item[0]) <= self.time:
						self.ships.append(Ship(item[1], item[2], item[3], item[4]))
						self.map.list.remove(item)
			
				for ship in self.ships:
					if ship.x > -30:
						if ship.update():
							if random.randint(0, 1) == 1:
								self.proiettili.append(Proiettile(-1, ship.weapon, (ship.x-ship.rect[0]-10, ship.y), 'enemy'))
						
						ship.draw(self.screen)
						
						if self.shipgap == 0:
							self.fumi.append(Fumo((ship.x, ship.y), 10))
					else:
						self.ships.remove(ship)
				
				
				# Movimento giocatore
				if self.playermove < 0 and self.playerpos > 40:
					self.playerpos += self.playermove
				if self.playermove > 0 and self.playerpos < 620:
					self.playerpos += self.playermove
			
				self.player.draw(self.screen, self.playerpos)
			
				# Interfaccia
				per = float(self.player.defense) / float(self.player.maxdefense)
				
				if per >= 0.6:
					color = (120, 220, 0)
				elif 0.30 <= per < 0.6:
					color = (255, 130, 0)
				elif per < 0.3:
					color = (220, 0, 0)
				
				pygame.draw.rect(self.screen, (50, 50, 50), (0, 0, 800, 30))
				pygame.draw.rect(self.screen, (0, 0, 0), (50, 5, 302, 20))
				pygame.draw.rect(self.screen, color, (51, 6, self.player.defense*self.player.perdef, 18))
			
				self.screen.blit(self.font.render(str(self.player.punti), True, (200, 200, 200)), (400, 3))
			
			if self.win:
				if self.wintime <= 0:
					self.active = False
					self.screen.blit(self.winfont.render('You win', True, (200, 200, 200)), (530, 530))
					
					if self.mode == 'camp' and not self.alreadywin:
						f = open('accounts/'+self.acc+'.acc')
						data = f.read().split('\n')
						f.close()
						
						f = open('accounts/'+self.acc+'.acc', 'w')
						part1 = data[0]+'\n'+data[1]+'\n'+data[2]+'\n'
						part2 = str(self.player.punti)+'\n'+str(int(data[4])+1)+'\n'+str(int(data[5])+self.player.punti-self.punti)
						f.write(part1+part2)
						f.close()
						
						pygame.image.save(pygame.transform.scale(self.screen, (150, 110)), 'accounts/'+self.acc+'.png')
					
					self.alreadywin = True
				else:
					self.wintime -= 1
			
			# Vittoria
			if self.map.list == [] and self.ships == []:
				self.win = True
			
			# Ultime cose
			pygame.display.flip()
			
			self.time += 25
			
			self.shipgap += 1
			self.fumigap += 1
			
			pygame.time.delay(25)
